// $Id$
//
// Copyright (C) 2008 

/** \file
    \brief UDPForwarder non-inline non-template implementation */

#include "UDPForwarder.hh"
//#include "UDPForwarder.ih"

// Custom includes

//#include "UDPForwarder.mpp"
#define prefix_
///////////////////////////////cc.p////////////////////////////////////////

prefix_ udpforward::UDPForwarder::UDPForwarder(senf::INet4SocketAddress const & addr,
                                               senf::INet4SocketAddress const & target)
    :  socket_ (addr), target_(target), peer_ (), 
       event_ ("UDPForwarder::read", senf::membind(&UDPForwarder::readEvent, this),
               socket_, senf::scheduler::FdEvent::EV_READ)
{
    SENF_LOG(("Input socket opened at " << addr << ". Forwarding to " << target));
}

prefix_ void udpforward::UDPForwarder::readEvent(int event)
{
    if (event != senf::scheduler::FdEvent::EV_READ) 
        throw ForwarderException("Unexpected remote socket read failure");

    std::string packet;
    senf::INet4SocketAddress addr;
    socket_.readfrom(packet, addr, 0);

    if (addr == target_) {
        if (! peer_)
            SENF_LOG((senf::log::MESSAGE)("Return packet dropped (no peer)"));
        else {
            SENF_LOG((senf::log::VERBOSE)("Return packet to " << peer_));
            socket_.writeto(peer_, packet);
        }
        return;
    }
    
    if (addr != peer_) {
        SENF_LOG((senf::log::NOTICE)("Peer address changed from " << peer_ << " to " << addr));
        peer_ = addr;
    }
    socket_.writeto(target_, packet);
    SENF_LOG((senf::log::VERBOSE)("Forward packet from " << peer_));
}

///////////////////////////////cc.e////////////////////////////////////////
#undef prefix_
//#include "UDPForwarder.mpp"


// Local Variables:
// mode: c++
// fill-column: 100
// indent-tabs-mode: nil
// compile-command: "scons"
// comment-column: 40
// End:
